/*
 *   Copyright 2007 Akhmad Fathonih <akhmadf@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include "cineplex21engine.h"

#include <QDate>
#include <QTime>
#include <QTimer>

#include <KDebug>
#include <KLocale>
#include <KSystemTimeZones>
#include <KDateTime>

#include "plasma/datacontainer.h"

class Cineplex21Engine::Private : public QObject
{
public:
    Private() {}
    ~Private() {}

private:
    struct XMLMapInfo {
        QString place;
        QString XMLurl;
        bool ukPlace;
    };

public:
    // Key dicts
//     QHash<QString, UKMETIon::Private::XMLMapInfo> m_place;
//     QVector<QString> m_locations;
public:

    // Store KIO jobs - Search list
    QMap<KJob *, QXmlStreamReader*> m_jobXml;
    QMap<KJob *, QString> m_jobList;

    KUrl *m_url;
    KIO::TransferJob *m_job;
};


Cineplex21Engine::Cineplex21Engine(QObject* parent, const QVariantList& args)
    : Plasma::DataEngine(parent), d(new Private())
{
    Q_UNUSED(args);
    setMinimumUpdateInterval(333);
}

Cineplex21Engine::~Cineplex21Engine()
{
}

bool Cineplex21Engine::sourceRequested(const QString &name)
{
    kDebug() << "Cineplex21Engine::sourceRequested " << name;
//     setData(name, I18N_NOOP("Movie"), "List schedule for given Theater ID");
//     if (name != "") {
//         setData("Theater:238", DataEngine::Data());
//         return true;
//     }
// 
//     return false;
    setData(name, DataEngine::Data());
    updateSource(name);
    return true;
}

// void Cineplex21Engine::updated(const QString& source, Plasma::DataEngine::Data data)
// {
//     setData(source, data);
// }

bool Cineplex21Engine::updateSource(const QString &source)
{
    kDebug() << "Cineplex21Engine::updateSource() " << source.section(":", 0, 0) << " " << source.section(":", 1, 1);
    
        
    if (source.section(":", 0, 0) != "Theater") 
        return false;
            
    bool OK;
    int TheaterID = source.section(":", 1, 1).toInt(&OK);
    
    if (!OK)
        return false;
        
    kDebug() << "Valid number: " << TheaterID;

    KUrl url;
//     url = "http://localhost/?TheaterID=" + source;
    url = QString("http://www.dapper.net/RunDapp?dappName=CINEPLEX21Version3&v=1&variableArg_0=%1").arg(TheaterID);
    
    if (source == "Test")
        url = "file:///home/kde-devel/RunDapp.xml";

    kDebug() << "URL: " << url;

    d->m_job = KIO::get(url.url(), true, false);
    d->m_jobXml.insert(d->m_job, new QXmlStreamReader);
    d->m_jobList.insert(d->m_job, source);
    
    //setData(source, I18N_NOOP("Movie"), "Updating source");
    
    if (d->m_job) {
        connect(d->m_job, SIGNAL(data(KIO::Job *, const QByteArray &)), this,
                SLOT(slotDataArrived(KIO::Job *, const QByteArray &)));
        connect(d->m_job, SIGNAL(result(KJob *)), this, SLOT(slotJobFinished(KJob *)));
    }

    return true;
}

void Cineplex21Engine::slotDataArrived(KIO::Job *job, const QByteArray &data)
{
    kDebug() << "Data arrived: " << data.count() << " bytes";
    if (data.isEmpty() || !d->m_jobXml.contains(job)) {
        return;
    }

    // Send to xml.
    d->m_jobXml[job]->addData(data.data());
}

void Cineplex21Engine::slotJobFinished(KJob *job)
{
    kDebug() << "Job finished";
    // reset movie count
    m_movieCount = 0;
    readSearchXMLData(d->m_jobList[job], *(d->m_jobXml[job]));
    d->m_jobList.remove(job);
//     delete m_jobXml[job];
//     m_jobXml.remove(job);
}

bool Cineplex21Engine::readSearchXMLData(QString key, QXmlStreamReader& xml)
{
    kDebug() << "readSearchXMLData";
    while (!xml.atEnd()) {
        xml.readNext();
 
        if (xml.isEndElement()) {
            break;
        }
 
        if (xml.isStartElement()) {
            kDebug() << "XML TAG: " << xml.name().toString();
            if (xml.name() == "Theatre_Name") {
                setData(key, "Theater.Name", xml.readElementText());
            } else 
            if (xml.name() == "movie_schedule_strip") {
                kDebug() << "Yatta~";
                parseMovieStrip(key, xml);
            } else {
//                 parseUnknownElement(xml);
                readSearchXMLData(key, xml);
            }
        }
    }

    return !xml.error();
}

void Cineplex21Engine::parseMovieStrip(QString key, QXmlStreamReader& xml)
{ 
    Q_UNUSED(key);
    kDebug() << "parseMovie"; 
    QString MovieTitle;
    m_scheduleCount = 0;
    
    Q_ASSERT(xml.isStartElement() && xml.name() == "movie_schedule_strip");
   
    while (!xml.atEnd()) {
        xml.readNext();
  
        if (xml.isEndElement() && xml.name() == "movie_schedule_strip") {
           break;
        }
     
        if (xml.isStartElement()) {
            if (xml.name() == "Movie_Title") {
                MovieTitle = xml.readElementText();
                setData(key, "Movie." + QString::number(m_movieCount) + ".Title" , MovieTitle);
                setData(key, "Movie.Count", m_movieCount + 1); // moviecount starts from zero
            }
            
            if (xml.name() == "studio_no") {
                setData(key, "Movie." + QString::number(m_movieCount) + ".Studio.No", xml.readElementText());
            }
            
            if (xml.name().toString().startsWith("schedule_")) {
                //setData(MovieTitle, xml.name().toString(), xml.readElementText());
                setData(key, "Movie." + QString::number(m_movieCount) + ".Schedule." + QString::number(m_scheduleCount), xml.readElementText());
                m_scheduleCount++;
                setData(key, "Movie." + QString::number(m_movieCount) + ".Schedule.Count", m_scheduleCount);
            }
        } 
    }
    m_movieCount++;
}

// handle when no XML tag is found
void Cineplex21Engine::parseUnknownElement(QXmlStreamReader& xml)
{

    while (!xml.atEnd()) {
        xml.readNext();

        if (xml.isEndElement()) {
            break;
        }

        if (xml.isStartElement()) {
            parseUnknownElement(xml);
        }
    }
}

#include "cineplex21engine.moc"
