/*
 *   Copyright 2007 Aaron Seigo <aseigo@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2 or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef TIMEENGINE_H
#define TIMEENGINE_H

#include <QtXml/QXmlStreamReader>
#include <QtCore/QStringList>
#include <QDebug>
#include <kurl.h>
#include <kio/job.h>
#include <kio/scheduler.h>
#include "plasma/dataengine.h"

class QTimer;

/**
 * This class evaluates the basic expressions given in the interface.
 */
class Cineplex21Engine : public Plasma::DataEngine
{
    Q_OBJECT

    public:
        Cineplex21Engine(QObject* parent, const QVariantList& args);
        ~Cineplex21Engine();

    protected:
        bool sourceRequested(const QString &theater);
        bool updateSource(const QString& source);
        
    protected slots:
        void slotDataArrived(KIO::Job *, const QByteArray &);
        void slotJobFinished(KJob *);
        
    private:
        int m_movieCount;
        int m_scheduleCount;
        bool readSearchXMLData(QString key, QXmlStreamReader& xml);
        void parseMovieStrip(QString key, QXmlStreamReader& xml);
        void parseUnknownElement(QXmlStreamReader& xml);
        
    private:
        class Private;
        Private *const d;
};

K_EXPORT_PLASMA_DATAENGINE(cineplex21, Cineplex21Engine)

#endif
